package dk.nsp.xdscleanup.deprecated.stack.registry.submissionsets;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.xdscleanup.common.model.SubmissionSet;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.deprecated.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;

import java.util.Collection;
import java.util.List;
import java.util.stream.Collectors;

public class DanglingSubmissionSetsCleanupEntryuuidPrefixSupplier implements StackOperation {

    private final RegistryDeletionService registryDeletionService;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final int limit;

    private final String entryuuidPrefix;

    public DanglingSubmissionSetsCleanupEntryuuidPrefixSupplier(RegistryDeletionService registryDeletionService,
                                                                CleanupOperationsFactory cleanupOperationsFactory,
                                                                int batchsize,
                                                                String entryuuidPrefix,
                                                                int limit) {
        this.registryDeletionService = registryDeletionService;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchsize;
        this.entryuuidPrefix = entryuuidPrefix;
        this.limit = limit;
    }


    @Override
    public List<StackOperation> run() {
        List<SubmissionSet> submissionSetsForDeletion = getSubmissionSetsForDeletion(entryuuidPrefix, limit);
        Collection<List<SubmissionSet>> entryBatches = CleanupUtils.createShuffledBatches(submissionSetsForDeletion, batchSize);

        return entryBatches.stream()
                .map(batch -> cleanupOperationsFactory.createRegistrySubmissionSetsBatchCleanupOperation(batch).withOperationName("Registry Dangling SubmissionSets Cleanup"))
                .collect(Collectors.toList());

    }

    private List<SubmissionSet> getSubmissionSetsForDeletion(String entryuuidPrefix, int limit) {
        try {
            return registryDeletionService.getDanglingSubmissionSetsForDeletion(entryuuidPrefix, limit);
        } catch (Exception e) {
            throw new CleanupException(String.format("Failed to retrieve dangling submissionsets"), e);
        }
    }


}
