package dk.nsp.xdscleanup.service;

import java.time.Duration;
import java.time.OffsetDateTime;
import java.time.ZoneId;
import java.util.concurrent.ConcurrentLinkedQueue;

public class ServiceErrorCallback {
    private final Duration errorCountDuration;
    private final ConcurrentLinkedQueue<OffsetDateTime> errorTimestamps = new ConcurrentLinkedQueue<>();
    private final ZoneId zoneId;

    public ServiceErrorCallback(Duration errorCountDuration, ZoneId zoneId) {
        this.errorCountDuration = errorCountDuration;
        this.zoneId = zoneId;
    }

    public void registerServiceError() {
        errorTimestamps.add(OffsetDateTime.now(zoneId));
    }

    public void registerServiceSuccess() {
        errorTimestamps.clear();
    }

    public int getErrorCount() {
        OffsetDateTime from = OffsetDateTime.now(zoneId).minus(errorCountDuration);
        errorTimestamps.removeIf(timestamp -> timestamp.isBefore(from));
        return errorTimestamps.size();
    }
}
