package dk.nsp.xdscleanup.integrations;

import dk.nsp.xdscleanup.service.ServiceErrorCallback;
import org.junit.Test;

import java.time.Duration;
import java.time.ZoneId;

import static org.junit.Assert.assertEquals;

public class ServiceErrorCallbackTest {

    private final static ZoneId zoneId = ZoneId.of("Europe/Copenhagen");

    @Test
    public void getErrorCount_WhenNothingHasBeenRegistered_Returns0() {
        ServiceErrorCallback serviceErrorCallback = new ServiceErrorCallback(Duration.ofMinutes(10), zoneId);

        int errorCount = serviceErrorCallback.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenSuccessHasBeenRegistered_Returns0() {
        ServiceErrorCallback serviceErrorCallback = new ServiceErrorCallback(Duration.ofMinutes(10), zoneId);
        serviceErrorCallback.registerServiceSuccess();

        int errorCount = serviceErrorCallback.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredWithinTimeLimit_Returns2() {
        ServiceErrorCallback serviceErrorCallback = new ServiceErrorCallback(Duration.ofMinutes(10), zoneId);
        serviceErrorCallback.registerServiceError();
        serviceErrorCallback.registerServiceError();

        int errorCount = serviceErrorCallback.getErrorCount();

        assertEquals(2, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredBeforeTimeLimit_Returns0() {
        ServiceErrorCallback serviceErrorCallback = new ServiceErrorCallback(Duration.ofMinutes(-1), zoneId);
        serviceErrorCallback.registerServiceError();
        serviceErrorCallback.registerServiceError();

        int errorCount = serviceErrorCallback.getErrorCount();

        assertEquals(0, errorCount);
    }

    @Test
    public void getErrorCount_WhenTwoErrorsHasBeenRegisteredWithinTimeLimitButSuccessAfterwards_Returns0() {
        ServiceErrorCallback serviceErrorCallback = new ServiceErrorCallback(Duration.ofMinutes(10), zoneId);
        serviceErrorCallback.registerServiceError();
        serviceErrorCallback.registerServiceError();
        serviceErrorCallback.registerServiceSuccess();

        int errorCount = serviceErrorCallback.getErrorCount();

        assertEquals(0, errorCount);
    }
}