package dk.nsp.xdscleanup.factory;

import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.model.DocumentTypeCleanupSpecification;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.openxds.service.RepositoryDeletionService;
import dk.nsp.xdscleanup.stack.registry.RegistryBatchCleanupOperation;
import dk.nsp.xdscleanup.stack.registry.deceased.PersonInformationService;
import dk.nsp.xdscleanup.stack.registry.deceased.DeceasedRegistryPatientIdBatchCleanupSupplier;
import dk.nsp.xdscleanup.stack.registry.deceased.DeceasedRegistryPatientIdPrefixCleanupSupplier;
import dk.nsp.xdscleanup.stack.registry.timebased.TypeAndTimeBasedRegistryCleanupSupplier;
import dk.nsp.xdscleanup.stack.repository.RepositoryBatchCleanupOperation;

import java.time.Period;
import java.util.List;

public class CleanupOperationsFactory {
    private final RegistryDeletionService registryDeletionService;
    private final RepositoryDeletionService repositoryDeletionService;
    private final DateTimeProvider dateTimeProvider;
    private final int registryBatchSize;
    private final Period deceasedPeriod;
    private final int registryTimebasedLimit;

    private final PersonInformationService personInformationService;

    public CleanupOperationsFactory(RegistryDeletionService registryDeletionService, RepositoryDeletionService repositoryDeletionService,
                                    DateTimeProvider dateTimeProvider, int registryBatchSize, Period deceasedPeriod, int registryTimebasedLimit, PersonInformationService personInformationService) {
        this.registryDeletionService = registryDeletionService;
        this.repositoryDeletionService = repositoryDeletionService;
        this.dateTimeProvider = dateTimeProvider;
        this.registryBatchSize = registryBatchSize;
        this.deceasedPeriod = deceasedPeriod;
        this.registryTimebasedLimit = registryTimebasedLimit;
        this.personInformationService = personInformationService;
    }

    public RegistryBatchCleanupOperation createRegistryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion) {
        return new RegistryBatchCleanupOperation(registryDeletionService, documentEntriesForDeletion, dateTimeProvider);
    }

    public TypeAndTimeBasedRegistryCleanupSupplier createTypeAndTimeBasedRegistryCleanupSupplier(DocumentTypeCleanupSpecification typeSpecification) {
        return new TypeAndTimeBasedRegistryCleanupSupplier(typeSpecification, registryDeletionService, dateTimeProvider, registryBatchSize, this, registryTimebasedLimit);
    }

    public DeceasedRegistryPatientIdBatchCleanupSupplier createDeceasedRegistryPatientIdBatchCleanupSupplier(List<String> patientIds) {
        return new DeceasedRegistryPatientIdBatchCleanupSupplier(patientIds, registryDeletionService, dateTimeProvider, this, registryBatchSize, deceasedPeriod, personInformationService);
    }

    public DeceasedRegistryPatientIdPrefixCleanupSupplier createDeceasedRegistryPatientIdPrefixCleanupSupplier(String patientIdPrefix) {
        return new DeceasedRegistryPatientIdPrefixCleanupSupplier(registryDeletionService, this, registryBatchSize, patientIdPrefix);
    }

    public RepositoryBatchCleanupOperation createRepositoryBatchCleanupOperation(List<DocumentEntry> documentEntriesForDeletion) {
        return new RepositoryBatchCleanupOperation(documentEntriesForDeletion, dateTimeProvider, repositoryDeletionService, registryDeletionService);
    }
}
