package dk.nsp.xdscleanup.stack.registry.deceased;

import dk.nsp.backgroundjob.stack.StackOperation;
import dk.nsp.backgroundjob.util.DateTimeProvider;
import dk.nsp.xdscleanup.common.model.DocumentEntry;
import dk.nsp.xdscleanup.common.util.CleanupUtils;
import dk.nsp.xdscleanup.factory.CleanupOperationsFactory;
import dk.nsp.xdscleanup.nxrg.service.RegistryDeletionService;
import dk.nsp.xdscleanup.common.util.CleanupException;

import java.time.Instant;
import java.time.Period;
import java.util.List;
import java.util.stream.Collectors;

public class DeceasedRegistryPatientIdBatchCleanupSupplier implements StackOperation {
    private final List<String> cprs;
    private final RegistryDeletionService registryDeletionService;
    private final DateTimeProvider dateTimeProvider;
    private final CleanupOperationsFactory cleanupOperationsFactory;
    private final int batchSize;
    private final Period deceasedPeriod;
    private final PersonInformationService personInformationService;

    public DeceasedRegistryPatientIdBatchCleanupSupplier(List<String> cprs, RegistryDeletionService registryDeletionService,
                                                         DateTimeProvider dateTimeProvider, CleanupOperationsFactory cleanupOperationsFactory, int batchSize,
                                                         Period deceasedPeriod, PersonInformationService personInformationService) {
        this.cprs = cprs;
        this.registryDeletionService = registryDeletionService;
        this.dateTimeProvider = dateTimeProvider;
        this.cleanupOperationsFactory = cleanupOperationsFactory;
        this.batchSize = batchSize;
        this.deceasedPeriod = deceasedPeriod;
        this.personInformationService = personInformationService;
    }

    @Override
    public List<StackOperation> run() {
        Instant latestDeceasedTime = dateTimeProvider.now().minus(deceasedPeriod).toInstant();
        List<String> deceasedCitizens = getDeceasedCitizens(latestDeceasedTime);
        List<DocumentEntry> documentEntriesForDeletion = getDocumentEntriesForCitizens(deceasedCitizens);

        return CleanupUtils.createShuffledBatches(documentEntriesForDeletion, batchSize).stream()
                .map(batch -> cleanupOperationsFactory.createRegistryBatchCleanupOperation(batch).withOperationName("Registry Deceased Cleanup"))
                .collect(Collectors.toList());
    }

    private List<String> getDeceasedCitizens(Instant latestDeceasedTime) {
        try {
            List<String> result = personInformationService.filterToDeceased(cprs, latestDeceasedTime);
            return result;
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve deceased citizens with latestStatusTime " + latestDeceasedTime, e);
        }
    }

    private List<DocumentEntry> getDocumentEntriesForCitizens(List<String> deceasedCitizens) {
        try {
            return registryDeletionService.getDocumentEntriesForCitizens(deceasedCitizens);
        } catch (Exception e) {
            throw new CleanupException("Failed to retrieve document entries for citizens", e);
        }
    }
}
