package dk.nsp.xdscleanup.stack.registry.deceased;

import dk.nsp.backgroundjob.health.HealthIndication;
import dk.nsp.backgroundjob.health.HealthIndicator;
import dk.nsp.xdscleanup.common.util.CleanupException;
import dk.nsp.xdscleanup.integrations.personinformation.PersonInformationClient;
import dk.nsp.xdscleanup.integrations.personinformation.PersonInformationClientException;

import java.time.Instant;

import java.util.List;

public class PersonInformationService  implements HealthIndicator {

    private final int errorTolerance;

    private final PersonInformationClient personInformationClient;

    public PersonInformationService(PersonInformationClient personInformationClient, int errorTolerance) {
        this.personInformationClient = personInformationClient;
        this.errorTolerance = errorTolerance;
    }

    public List<String> filterToDeceased(List<String> cprs, Instant deceasedSince)  {

        try {
            return personInformationClient.personDeceased(cprs, deceasedSince);
        } catch (PersonInformationClientException e) {
            throw new CleanupException("Failed to call PatientInformation", e);
        }
    }

    @Override
    public HealthIndication getHealthInfo() {
        String serviceName = "PersonInformation";
        if (personInformationClient.getErrorCount() > errorTolerance) {
            return new HealthIndication(500, "\""+serviceName+"\": \"ERROR\"");
        } else {
            return new HealthIndication(200, "\""+serviceName+"\": \"OK\"");

        }

    }
}
