package dk.nsp.xdscleanup.stack.registry.deceased;


import dk.nsp.xdscleanup.integrations.personinformation.PersonInformationClient;
import dk.nsp.xdscleanup.integrations.personinformation.PersonInformationClientException;

import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import java.time.Instant;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collections;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;
import static org.mockito.Matchers.any;
import static org.mockito.Mockito.doAnswer;

public class PersonInformationServiceTest {

    private PersonInformationService subject;
    private PersonInformationClient personInformationClient;
    @Before
    public void setUp() {
        personInformationClient = Mockito.mock(PersonInformationClient.class);
        subject = new PersonInformationService(personInformationClient, 0);
    }

    @Test
    public void getDeceasedCitizens_WithNonExistingCitizen_ReturnsEmptyList() throws PersonInformationClientException {

        // Arrange
        String nonExistingCitizen = "0000000000";
        List<String> responseCprs = new ArrayList<>();
        doAnswer(arg -> responseCprs).when(personInformationClient).personDeceased(any(List.class), any(Instant.class));

        // Act
        List<String> result = subject.filterToDeceased(Collections.singletonList(nonExistingCitizen), Instant.now());

        // Assert
        assertTrue(result.isEmpty());
    }

    @Test
    public void getDeceasedCitizens_WithEmptyCitizenList_ReturnsEmptyList() throws PersonInformationClientException {

        // Arrange
        List<String> responseCprs = new ArrayList<>();
        doAnswer(arg -> responseCprs).when(personInformationClient).personDeceased(any(List.class), any(Instant.class));

        // Act
        List<String> result = subject.filterToDeceased(Collections.emptyList(), Instant.now());

        // Assert
        assertTrue(result.isEmpty());

    }

    @Test
    public void getDeceasedCitizens_WithOneOutOfThreeDeceasedCitizens_ReturnsDeceasedCitizen() throws PersonInformationClientException {

        // Arrange
        String nonExistingCitizen = "0000000000";
        String deceasedCitizen = "0000000090";
        String futureDeceasedCitizen = "0000000091";
        List<String> responseCprs = Arrays.asList(new String[]{deceasedCitizen});
        doAnswer(arg -> responseCprs).when(personInformationClient).personDeceased(any(List.class), any(Instant.class));

        // Act
        List<String> result = subject.filterToDeceased(Arrays.asList(nonExistingCitizen, deceasedCitizen, futureDeceasedCitizen), Instant.now());

        // Assert
        assertEquals(1, result.size());

    }

    @Test
    public void getDeceasedCitizens_WithThreeOutOfThreeDeceasedCitizens_ReturnsDeceasedCitizen() throws PersonInformationClientException {

        // Arrange
        String deceasedCitizen1 = "0000000091";
        String deceasedCitizen2 = "0000000092";
        String deceasedCitizen3 = "0000000093";
        List<String> responseCprs = Arrays.asList(new String[]{deceasedCitizen1, deceasedCitizen2, deceasedCitizen3});
        doAnswer(arg -> responseCprs).when(personInformationClient).personDeceased(any(List.class), any(Instant.class));

        // Act
        List<String> result = subject.filterToDeceased(Arrays.asList(deceasedCitizen1, deceasedCitizen2, deceasedCitizen3), Instant.now());

        // Assert
        assertEquals(3, result.size());
    }

}
